//
//  ViewController.m
//  iPhoneCV2
//
//  Created by Pablo Roldán Ruz on 26/10/11.
//  Copyright (c) 2011 __MyCompanyName__. All rights reserved.
//

#import "ViewController.h"
#import <CoreImage/CoreImage.h>
#import <ImageIO/ImageIO.h>
#import <AssertMacros.h>
#import <AssetsLibrary/AssetsLibrary.h>
#import "ImageUtils.m"


// NEON SIMD color to grayscale conversion
extern void neon_to_gray (cv::Mat &image, cv::Mat &grayMatImage);

// Face cascade classifier file name
static NSString *kFaceCascadeFilename = @"haarcascade_frontalface_alt2";

// used for KVO observation of the @"capturingStillImage" property to perform flash animation
static const NSString *AVCaptureStillImageIsCapturingStillImageContext = @"AVCaptureStillImageIsCapturingStillImageContext";

// used for KVO observation of the number of people in the pictre to update the label
static const NSString *SettingsControllerNumberOfPeopleContext = @"SettingsControllerNumberOfPeopleContext";


// Main app controller private methods
@interface ViewController()

// Capture session setup.
- (void)setupCaptureSession;

// Creates a cv::Mat image from a CMSampleBufferRef image
- (void)createMatImageFromSampleBuffer:(CMSampleBufferRef)sampleBuffer;

// Creates a CGImageRef from a cv::Mat image
- (CGImageRef)createCGImageFromMatImage:(cv::Mat&)image;

// Update the video orientation
- (void)updateVideoOrientation;

// Gets the camera with the position 'position' (back or front)
- (AVCaptureDevice *) cameraWithPosition:(AVCaptureDevicePosition) position;

// Gets the front camera
- (AVCaptureDevice *) frontFacingCamera;

// Gets the back camera
- (AVCaptureDevice *) backFacingCamera;

// Close the capture session
- (void)teardownAVCapture;

@end


// Main app controller implementation
@implementation ViewController

@synthesize cameraView;
@synthesize detectionView;
@synthesize captureSession;
@synthesize previewLayer;
@synthesize videoInput;
@synthesize peopleLabel;
@synthesize settingsController;
@synthesize takeNewPictureButton;


- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Release any cached data, images, etc that aren't in use.
}


#pragma mark - View lifecycle
// Initialize the app when the main view is loaded
- (void)viewDidLoad
{    
    [super viewDidLoad];
    
    // Deactivate animations
    [UIView setAnimationsEnabled:NO];
    
    // Init the capture session
    [self setupCaptureSession];
    
    // View aspect
    self.detectionView.frame = cameraView.frame;
    self.detectionView.bounds = cameraView.bounds;
    self.detectionView.layer.contentsGravity = kCAGravityResizeAspect;
    
    if (self.captureSession) {
        // Display the preview from the camera
        AVCaptureVideoPreviewLayer *newCaptureVideoPreviewLayer = [[AVCaptureVideoPreviewLayer alloc] initWithSession:self.captureSession];
        
        [self.cameraView.layer setMasksToBounds:YES];
        
        [newCaptureVideoPreviewLayer setFrame:self.cameraView.bounds];
                
        newCaptureVideoPreviewLayer.contentsGravity = kCAGravityResizeAspect;
        [newCaptureVideoPreviewLayer setVideoGravity: AVLayerVideoGravityResize];
        
        // Add preview layer to the view
        [self.cameraView.layer insertSublayer:newCaptureVideoPreviewLayer below:[[self.cameraView.layer sublayers] objectAtIndex:0]];
        
        self.previewLayer = newCaptureVideoPreviewLayer;
        [newCaptureVideoPreviewLayer release];  
        
        // Update the video orientation
        [self updateVideoOrientation];
        
        // KVO observer
        [settingsController addObserver:self forKeyPath:@"numberOfPeople" options:NSKeyValueObservingOptionNew context:(void *)SettingsControllerNumberOfPeopleContext];
        
        // Start capture session
        [self.captureSession startRunning];
    }
    
    
    // Load face cascade classifier
    NSString *faceCascadePath = [[NSBundle mainBundle] pathForResource:kFaceCascadeFilename ofType:@"xml"];
    
    if (!faceCascade.load([faceCascadePath UTF8String])) {
        NSLog(@"Could not load face cascade: %@", faceCascadePath);
    }    
    
    scale = 3.25;
    
    // Init the number of people detected label
    self.settingsController.numberOfPeople = [NSNumber numberWithInt:2];    
    
    pictureTaken = YES;
    numberOfPeopleDetected = 0;
    
    self.peopleLabel.text = [NSString stringWithFormat:@"%d/%d people in the picture",numberOfPeopleDetected,[self.settingsController.numberOfPeople intValue]];
}

- (void)viewDidUnload
{
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
    
    [self.captureSession stopRunning];
        
    faceCascade.~CascadeClassifier();
    
    self.cameraView = nil;
    self.detectionView = nil;
    self.captureSession = nil;
    self.previewLayer = nil;
    self.videoInput = nil;
    self.peopleLabel = nil;
    self.settingsController = nil;
    self.takeNewPictureButton = nil;
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
}

- (void)viewWillDisappear:(BOOL)animated
{
	[super viewWillDisappear:animated];
}

- (void)viewDidDisappear:(BOOL)animated
{
	[super viewDidDisappear:animated];
}


// Close the capture session
- (void)teardownAVCapture {
    [stillImageOutput removeObserver:self forKeyPath:@"isCapturingStillImage"];
	[stillImageOutput release];
    
    if (cameraQueue)
		dispatch_release(cameraQueue);    
    
    [captureSession release];
    [videoInput release];
    [videoOutput release];
}

- (void)dealloc {
    [self teardownAVCapture];

    [cameraView release];
    [detectionView release];
    [previewLayer release];
    [peopleLabel release];
    [settingsController release];
    [takeNewPictureButton release];
        
    [super dealloc];
}


#pragma mark - Video Orientation
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    // Return YES for supported orientations
    return (interfaceOrientation != UIInterfaceOrientationPortraitUpsideDown);
}

- (void)didRotateFromInterfaceOrientation:(UIInterfaceOrientation)fromInterfaceOrientation {
    // Update the orientation of the video captured in the capture session
    [self updateVideoOrientation];
}


// Update the video orientation
- (void)updateVideoOrientation {
    
    for (AVCaptureOutput *out in self.captureSession.outputs) {  
        
        // Get the device orientation
        UIDeviceOrientation orientation = [[UIDevice currentDevice] orientation];
        
        switch (orientation) {
            case UIDeviceOrientationUnknown:
                if ([[out connectionWithMediaType:AVMediaTypeVideo] isVideoOrientationSupported]) {
                    [[out connectionWithMediaType:AVMediaTypeVideo] setVideoOrientation:AVCaptureVideoOrientationPortrait];          
                    
                    if ([[out connectionWithMediaType:AVMediaTypeVideo] videoOrientation] == AVCaptureVideoOrientationPortrait) {
                    }
                }
                [self.previewLayer setOrientation:AVCaptureVideoOrientationPortrait];
                break;
                
            case UIDeviceOrientationPortrait:
                if ([[out connectionWithMediaType:AVMediaTypeVideo] isVideoOrientationSupported]) {
                    [[out connectionWithMediaType:AVMediaTypeVideo] setVideoOrientation:AVCaptureVideoOrientationPortrait];                
                }
                [self.previewLayer setOrientation:AVCaptureVideoOrientationPortrait];
                break;
                
            case UIDeviceOrientationPortraitUpsideDown:
                [[out connectionWithMediaType:AVMediaTypeVideo] setVideoOrientation:AVCaptureVideoOrientationPortraitUpsideDown];
                [self.previewLayer setOrientation:AVCaptureVideoOrientationPortraitUpsideDown];
                break;
                
            case UIDeviceOrientationLandscapeLeft:
                [[out connectionWithMediaType:AVMediaTypeVideo] setVideoOrientation:AVCaptureVideoOrientationLandscapeRight];
                [self.previewLayer setOrientation:AVCaptureVideoOrientationLandscapeRight];
                break;
                
            case UIDeviceOrientationLandscapeRight:
                [[out connectionWithMediaType:AVMediaTypeVideo] setVideoOrientation:AVCaptureVideoOrientationLandscapeLeft];
                [self.previewLayer setOrientation:AVCaptureVideoOrientationLandscapeLeft];
                break;
                
            case UIDeviceOrientationFaceUp:
                [[out connectionWithMediaType:AVMediaTypeVideo] setVideoOrientation:AVCaptureVideoOrientationPortrait];
                [self.previewLayer setOrientation:AVCaptureVideoOrientationPortrait];
                break;
                
            case UIDeviceOrientationFaceDown:
                [[out connectionWithMediaType:AVMediaTypeVideo] setVideoOrientation:AVCaptureVideoOrientationPortrait];
                [self.previewLayer setOrientation:AVCaptureVideoOrientationPortrait];
                break;
                
            default:
                [[out connectionWithMediaType:AVMediaTypeVideo] setVideoOrientation:AVCaptureVideoOrientationPortrait];
                [self.previewLayer setOrientation:AVCaptureVideoOrientationPortrait];
                break;
        }
        
    }
    
    // Destroy the frame images. The orientation has changed and so did their size
    frameImage.release();
    grayImage.release();
    smallImage.release();
    [self.previewLayer setFrame:self.cameraView.bounds];
}

#pragma mark Setup capture session

// Initialize the capture session
- (void)setupCaptureSession {
    
    // Create the session
    self.captureSession = [[AVCaptureSession alloc] init];
    
    // Session resolution
    self.captureSession.sessionPreset = AVCaptureSessionPreset640x480; //AVCaptureSessionPresetMedium; 
    
    // Find a suitable AVCaptureDevice
    AVCaptureDevice *device = [AVCaptureDevice defaultDeviceWithMediaType:AVMediaTypeVideo];
    
    // Create a device input with the device and add it to the session
    NSError *error;
    self.videoInput = [AVCaptureDeviceInput deviceInputWithDevice:device error:&error];
    require( error == nil, bail );
    
    [self.captureSession addInput:videoInput];
    
    // Make a still image output	
    stillImageOutput = [AVCaptureStillImageOutput new];
        
	[stillImageOutput addObserver:self forKeyPath:@"capturingStillImage" options:NSKeyValueObservingOptionNew context:(void*)AVCaptureStillImageIsCapturingStillImageContext];
    
	if ([captureSession canAddOutput:stillImageOutput])
        [self.captureSession addOutput:stillImageOutput];
    
    
    // Create video data output and add it to the session
    videoOutput = [[AVCaptureVideoDataOutput alloc] init];

    
    // Specify the pixel format (BGRA is supposed to be faster)
    videoOutput.videoSettings = [NSDictionary dictionaryWithObject:[NSNumber numberWithInt:kCVPixelFormatType_32BGRA] forKey:(id)kCVPixelBufferPixelFormatTypeKey];
    videoOutput.alwaysDiscardsLateVideoFrames = YES;

    
    // Configure output
    cameraQueue = dispatch_queue_create("cameraQueue", DISPATCH_QUEUE_SERIAL);
    [videoOutput setSampleBufferDelegate:self queue:cameraQueue];
    //dispatch_release(queue);
    
    if([captureSession canAddOutput:videoOutput])
        [self.captureSession addOutput:videoOutput];

// Handle error
bail:
	[captureSession release];
	if (error) {
		UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:[NSString stringWithFormat:@"Failed with error %d", (int)[error code]]
															message:[error localizedDescription]
														   delegate:nil 
												  cancelButtonTitle:@"Dismiss" 
												  otherButtonTitles:nil];
		[alertView show];
		[alertView release];
		[self teardownAVCapture];
	}    
}

// Toggle between the front and back camera
- (IBAction) toggleCamera:(id) sender {
    
    dispatch_sync(cameraQueue, ^(void) {
        
        if ([[AVCaptureDevice devicesWithMediaType:AVMediaTypeVideo] count] > 1) {
            NSError *error;
            AVCaptureDeviceInput *newVideoInput;
            AVCaptureDevicePosition position = [[videoInput device] position];
        
            if (position == AVCaptureDevicePositionBack)
                newVideoInput = [[AVCaptureDeviceInput alloc] initWithDevice:[self frontFacingCamera] error:&error];
            else if (position == AVCaptureDevicePositionFront) 
                newVideoInput = [[AVCaptureDeviceInput alloc] initWithDevice:[self backFacingCamera] error:&error];
        
            if (newVideoInput != nil) {
                [self.captureSession stopRunning];
                [self.captureSession beginConfiguration];
                [self.captureSession removeInput:[self videoInput]];
                if ([self.captureSession canAddInput:newVideoInput]) {
                    [self.captureSession addInput:newVideoInput];
                    self.videoInput = newVideoInput;
                } else {
                    [self.captureSession addInput:self.videoInput];
                }
                [self.captureSession commitConfiguration];
                [newVideoInput release];
                [self updateVideoOrientation];
                [self.captureSession startRunning];
                self.detectionView.frontCamera = !self.detectionView.frontCamera;
                
            } 
        }
    });
}

// Find a camera with the specificed AVCaptureDevicePosition, returning nil if one is not found
- (AVCaptureDevice *) cameraWithPosition:(AVCaptureDevicePosition) position
{
    NSArray *devices = [AVCaptureDevice devicesWithMediaType:AVMediaTypeVideo];
    for (AVCaptureDevice *device in devices) {
        if ([device position] == position) {
            return device;
        }
    }
    return nil;
}


// Find a front facing camera, returning nil if one is not found
- (AVCaptureDevice *) frontFacingCamera
{
    return [self cameraWithPosition:AVCaptureDevicePositionFront];
}


// Find a back facing camera, returning nil if one is not found
- (AVCaptureDevice *) backFacingCamera
{
    return [self cameraWithPosition:AVCaptureDevicePositionBack];
}



#pragma mark - Mat Image Conversion

// Creates a cv::Mat image from a CMSampleBuffer image
- (void)createMatImageFromSampleBuffer:(CMSampleBufferRef)sampleBuffer {
    
    if (sampleBuffer) {
        // Get the CVImageBufferRef
        CVImageBufferRef imageBuffer = CMSampleBufferGetImageBuffer(sampleBuffer);
        // Lock base address
        CVPixelBufferLockBaseAddress(imageBuffer, 0);
        
        // get information of the image in the buffer
        void *bufferBaseAddress = CVPixelBufferGetBaseAddress(imageBuffer);
        size_t bufferWidth = CVPixelBufferGetWidth(imageBuffer);
        size_t bufferHeight = CVPixelBufferGetHeight(imageBuffer);
                
        // If frameImage hasn't been created yet, it's created now
        if (!frameImage.data) {
            frameImage = cv::Mat(bufferHeight, bufferWidth, CV_8UC4);
        }
        
        // Update data pointer
        if (bufferBaseAddress) {
            frameImage.data = (uchar*)bufferBaseAddress;
        }
        
        // release memory
        CVPixelBufferUnlockBaseAddress(imageBuffer, 0);
    }
}

// Create a CGImageRef from a cv::Mat image
- (CGImageRef)createCGImageFromMatImage:(cv::Mat&)image {
    CGColorSpaceRef colorSpace;
    CGImageRef imageRef;
    
    // Color image
    if (image.channels() > 1) {
        colorSpace = CGColorSpaceCreateDeviceRGB();
        CGDataProviderRef provider = CGDataProviderCreateWithData(NULL, image.data, image.elemSize()*image.total(), NULL);
        
        imageRef = CGImageCreate(image.cols, image.rows, 8, image.elemSize()*8, image.step[0],colorSpace, kCGImageAlphaNoneSkipFirst | kCGBitmapByteOrder32Little,provider, NULL, false, kCGRenderingIntentDefault);
        
        CGDataProviderRelease(provider);
    } else { // Grayscale image
        colorSpace = CGColorSpaceCreateDeviceGray();
        
        CGContextRef newContextGray = CGBitmapContextCreate(image.data, image.size().width, image.size().height, 8, image.size().width, colorSpace, kCGImageAlphaNone);
        
        //Create CgImageRef from cgcontext
        imageRef = CGBitmapContextCreateImage(newContextGray);
    }
    
    CGColorSpaceRelease(colorSpace);
    
    return imageRef;
}


#pragma mark - AVCaptureVideoDataOutput delegate

// Delegate method for capture session. This method is called every time a frame is captured by the camera
- (void)captureOutput:(AVCaptureOutput *)captureOutput didOutputSampleBuffer:(CMSampleBufferRef)sampleBuffer fromConnection:(AVCaptureConnection *)connection {

    if (!pictureTaken) {
        
        // Array results
        std::vector<cv::Rect> faces;
    
        // Process frame image
        [self createMatImageFromSampleBuffer:sampleBuffer];
        
        // If grayImage hasn't been created yet, it's created now
        if(!grayImage.data) {
            grayImage = cv::Mat(frameImage.size().height, frameImage.size().width, CV_8UC1);
            smallImage = cv::Mat(frameImage.size().height/scale, frameImage.size().width/scale, CV_8UC1);
        }
        
        // NEON SIMD grayscale conversion
        neon_to_gray(frameImage,grayImage);
        
        // Create a smaller version of the grayscale image
        cv::resize(grayImage, smallImage, cv::Size(smallImage.size().width,smallImage.size().height));
    
        // Face detection
        faceCascade.detectMultiScale(smallImage, 
                                     faces, 
                                     1.2f, 
                                     2, 
                                     0 | CV_HAAR_DO_CANNY_PRUNING,
                                     cv::Size(20, 20));
        
        // Draw a rectangle for each face detected
        float scaleWidth = self.cameraView.frame.size.width / frameImage.size().width;
        float scaleHeight = self.cameraView.frame.size.height / frameImage.size().height;

        CGMutablePathRef pathRef=CGPathCreateMutable();
    
        for (int i = 0; i < faces.size(); i++) {   
            CGPathAddRect(pathRef, NULL, CGRectMake(faces[i].x*scaleWidth*scale, faces[i].y*scaleHeight*scale, faces[i].width*scaleWidth*scale, faces[i].height*scaleHeight*scale));
        }
        
        // Update the number of people detected
        numberOfPeopleDetected = faces.size();
    
        // Update the label
        [self.peopleLabel performSelectorOnMainThread:@selector(setText:) withObject:[NSString stringWithFormat:@"%d/%d people in the picture",numberOfPeopleDetected,[self.settingsController.numberOfPeople intValue]] waitUntilDone:NO];
    
        // If the number of faces are greater or equal than the number of people set by the user, 
        // the picture is taken
        if (numberOfPeopleDetected >= [self.settingsController.numberOfPeople intValue] && pictureTaken == NO)
        {
            pictureTaken = YES;
            numberOfPeopleDetected = 0;
            dispatch_async(dispatch_get_main_queue(), ^(void) {
                   [self takePicture:nil];
            });
        }
    
        dispatch_async(dispatch_get_main_queue(), ^(void) {
            self.detectionView.pathToDraw = pathRef;
        });
    } else {
        dispatch_async(dispatch_get_main_queue(), ^(void) {
            self.detectionView.pathToDraw = nil;
            numberOfPeopleDetected = 0;
        });        
    }
        
}

#pragma mark - Take photo

// This method writes the resulting image to the camera roll
- (BOOL)writeCGImageToCameraRoll:(CGImageRef)cgImage withMetadata:(NSDictionary *)metadata
{
	CFMutableDataRef destinationData = CFDataCreateMutable(kCFAllocatorDefault, 0);
	CGImageDestinationRef destination = CGImageDestinationCreateWithData(destinationData, 
																		 CFSTR("public.jpeg"), 
																		 1, 
																		 NULL);
	BOOL success = (destination != NULL);
    
	const float JPEGCompQuality = 0.85f; // JPEGHigherQuality
	CFMutableDictionaryRef optionsDict = NULL;
	CFNumberRef qualityNum = NULL;
	
	qualityNum = CFNumberCreate(0, kCFNumberFloatType, &JPEGCompQuality);    
	if ( qualityNum ) {
		optionsDict = CFDictionaryCreateMutable(0, 0, &kCFTypeDictionaryKeyCallBacks, &kCFTypeDictionaryValueCallBacks);
		if ( optionsDict )
			CFDictionarySetValue(optionsDict, kCGImageDestinationLossyCompressionQuality, qualityNum);
		CFRelease( qualityNum );
	}
	
	CGImageDestinationAddImage( destination, cgImage, optionsDict );
	success = CGImageDestinationFinalize( destination );
    
	if ( optionsDict )
		CFRelease(optionsDict);
	
	CFRetain(destinationData);
	ALAssetsLibrary *library = [ALAssetsLibrary new];
	[library writeImageDataToSavedPhotosAlbum:(id)destinationData metadata:metadata completionBlock:^(NSURL *assetURL, NSError *error) {
		if (destinationData)
			CFRelease(destinationData);
	}];
	[library release];
    
	return success;
}

// KVO observer: 
//   - Performs a flash animation when the picture is taken
//   - Updates the number of people in the picture label
- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{
    // Picture taken
	if ( context == AVCaptureStillImageIsCapturingStillImageContext ) 
    {
		BOOL isCapturingStillImage = [[change objectForKey:NSKeyValueChangeNewKey] boolValue];
		
		if ( isCapturingStillImage ) {
			// do flash bulb like animation
			flashView = [[UIView alloc] initWithFrame:self.view.frame];
			[flashView setBackgroundColor:[UIColor whiteColor]];
			[flashView setAlpha:0.f];
			[[[self view] window] addSubview:flashView];
            
            
            NSError *error;
            
            NSString *newAudioFile = [[[NSBundle mainBundle] pathForResource:@"shutter" ofType:@"wav"] retain];
            
            
            AVAudioPlayer *audioPlayer = [[[AVAudioPlayer alloc] initWithContentsOfURL:[NSURL fileURLWithPath:newAudioFile] error:&error] retain];
            
            if(error) {
                
            }
            
            // Shutter sound
            [audioPlayer play];
            
            [newAudioFile release];
            [audioPlayer release];
    
			self.takeNewPictureButton.title = [NSString stringWithFormat:@"Take new picture"];
            
			[UIView animateWithDuration:.1f
							 animations:^{
								 [flashView setAlpha:1.f];
							 }
			 ];
		}
		else {
            
			[UIView animateWithDuration:.1f
							 animations:^{
								 [flashView setAlpha:0.f];
							 }
							 completion:^(BOOL finished){
								 [flashView removeFromSuperview];
								 [flashView release];
								 flashView = nil;
							 }
			 ];
		}
	}
    
    // Number of people in the picture label
    if (context == SettingsControllerNumberOfPeopleContext) {
        int people = [[change objectForKey:NSKeyValueChangeNewKey] intValue];
        self.peopleLabel.text = [NSString stringWithFormat:@"%d/%d people in the picture",numberOfPeopleDetected,people];
    }
}

// utility routing used during image capture to set up capture orientation
- (AVCaptureVideoOrientation)avOrientationForDeviceOrientation:(UIDeviceOrientation)deviceOrientation
{
	AVCaptureVideoOrientation result = deviceOrientation;
	if ( deviceOrientation == UIDeviceOrientationLandscapeLeft )
		result = AVCaptureVideoOrientationLandscapeRight;
	else if ( deviceOrientation == UIDeviceOrientationLandscapeRight )
		result = AVCaptureVideoOrientationLandscapeLeft;
	return result;
}

// utility routine to display error aleart if takePicture fails
- (void)displayErrorOnMainQueue:(NSError *)error withMessage:(NSString *)message
{
	dispatch_async(dispatch_get_main_queue(), ^(void) {
		UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:[NSString stringWithFormat:@"%@ (%d)", message, (int)[error code]] message:[error localizedDescription] 
                                                           delegate:nil 
                                                  cancelButtonTitle:@"Dismiss" 
												  otherButtonTitles:nil];
		[alertView show];
		[alertView release];
	});
}


// method to take a still image -- if face detection has been turned on and a face has been detected
- (IBAction)takePicture:(id)sender
{
	// Find out the current orientation and tell the still image output.
	AVCaptureConnection *stillImageConnection = [stillImageOutput connectionWithMediaType:AVMediaTypeVideo];
	UIDeviceOrientation curDeviceOrientation = [[UIDevice currentDevice] orientation];
	AVCaptureVideoOrientation avcaptureOrientation = [self avOrientationForDeviceOrientation:curDeviceOrientation];
	[stillImageConnection setVideoOrientation:avcaptureOrientation];
	[stillImageConnection setVideoScaleAndCropFactor:1.0];
	
    // set the appropriate pixel format / image type output setting depending on if we'll need an uncompressed image for
    // the possiblity of drawing the red square over top or if we're just writing a jpeg to the camera roll which is the trival case
	[stillImageOutput setOutputSettings:[NSDictionary dictionaryWithObject:[NSNumber numberWithInt:kCMPixelFormat_32BGRA] forKey:(id)kCVPixelBufferPixelFormatTypeKey]];
	
	[stillImageOutput captureStillImageAsynchronouslyFromConnection:stillImageConnection completionHandler:^(CMSampleBufferRef imageDataSampleBuffer, NSError *error) {        
        if (error) {
            [self displayErrorOnMainQueue:error withMessage:@"Take picture failed"];
        } else {
            // Got an image.
            CVPixelBufferRef pixelBuffer = CMSampleBufferGetImageBuffer(imageDataSampleBuffer);
            CFDictionaryRef attachments = CMCopyDictionaryOfAttachments(kCFAllocatorDefault,imageDataSampleBuffer, kCMAttachmentMode_ShouldPropagate);
                
            CIImage *ciImage = [[CIImage alloc] initWithCVPixelBuffer:pixelBuffer options:(NSDictionary *)attachments];
                
            if (attachments)
                CFRelease(attachments);
                
            NSDictionary *imageOptions = nil;
            NSNumber *orientation = (NSNumber *)CMGetAttachment(imageDataSampleBuffer, kCGImagePropertyOrientation, NULL);
                
            if (orientation) {
                imageOptions = [NSDictionary dictionaryWithObject:orientation forKey:CIDetectorImageOrientation];
            }
                
            // when processing an existing frame we want any new frames to be automatically dropped
            // queueing this block to execute on the videoDataOutputQueue serial queue ensures this
            // see the header doc for setSampleBufferDelegate:queue: for more information
            dispatch_sync(cameraQueue, ^(void) {
                CGImageRef srcImage = NULL;
                OSStatus err = CreateCGImageFromCVPixelBuffer(CMSampleBufferGetImageBuffer(imageDataSampleBuffer), &srcImage);
                    
                check(!err);                        
                
                CFDictionaryRef attachments = CMCopyDictionaryOfAttachments(kCFAllocatorDefault,
                                                                            imageDataSampleBuffer,
                                                                            kCMAttachmentMode_ShouldPropagate);
                
                pictureTaken = [self writeCGImageToCameraRoll:srcImage withMetadata:(id)attachments];
                    
                if (attachments)
                    CFRelease(attachments);
                
                if (srcImage)
                    CFRelease(srcImage);
            });
            
            [ciImage release];
        }
    }];
}

// Action for the settings button
- (IBAction)settings:(id)sender {
    settingsController.modalTransitionStyle = UIModalTransitionStyleCoverVertical;
    pictureTaken = YES;
    self.takeNewPictureButton.title = [NSString stringWithFormat:@"Take new picture"];  
    [self presentModalViewController:settingsController animated:YES];
}

// Action for the take new picture button
- (IBAction)newPicture:(id)sender {
    pictureTaken = NO;
    self.takeNewPictureButton.title = [NSString stringWithFormat:@"Taking picture (%d)",[self.settingsController.numberOfPeople intValue]];    
}

@end
