//
//  VideoPlayerCV.m
//  VideoTestCV
//
//  Created by Pablo Roldán Ruz on 21/11/11.
//  Copyright (c) 2011 __MyCompanyName__. All rights reserved.
//

#import "VideoPlayerCV.h"
#import "AVFoundation/AVFoundation.h"
#import "ResultSet.h"
#import "Frame.h"
#import "Detection.h"

// NEON SIMD grayscale converter
extern void neon_to_gray (cv::Mat &image, cv::Mat &grayMatImage);

// Cascade classifiers file names
NSString * const kFaceCascadeFilename = @"haarcascade_frontalface_alt2";
NSString * const kUpperBodyCascadeFilename = @"haarcascade_upperbody";
NSString * const kFullBodyCascadeFilename = @"haarcascade_fullbody";
NSString * const kLowerBodyCascadeFilename = @"haarcascade_lowerbody2";


/*
 * This class plays a video and shows the objects detected in its frames *
 */
@implementation VideoPlayerCV

@synthesize reader;
@synthesize assetReaderOutput;
@synthesize inputAsset;
@synthesize videoFile;
@synthesize fpsLabel;
@synthesize groundTruth;


// Init method
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    
    // Aspect ratio (Resize)
    self.view.layer.contentsGravity = kCAGravityResizeAspect;
    
    // Load the ground truth data you want to show in the video
    self.groundTruth = [[ResultSet alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"FaceGroundTruth" ofType:@"txt"]];  
    
    return self;
}

- (void)didReceiveMemoryWarning
{
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
}

#pragma mark - View lifecycle

// The classifiers are loaded when the view is loaded
- (void)viewDidLoad
{
    [super viewDidLoad];
    // Do any additional setup after loading the view from its nib.    
    
    // Get the cascade classifiers file path from resources
    NSString *faceCascadePath = [[NSBundle mainBundle] pathForResource:kFaceCascadeFilename ofType:@"xml"];
    NSString *upperBodyCascadePath = [[NSBundle mainBundle] pathForResource:kUpperBodyCascadeFilename ofType:@"xml"];
    NSString *fullBodyCascadePath = [[NSBundle mainBundle] pathForResource:kFullBodyCascadeFilename ofType:@"xml"];
    NSString *lowerBodyCascadePath = [[NSBundle mainBundle] pathForResource:kLowerBodyCascadeFilename ofType:@"xml"];
    
    // Load face cascade classifier
    if (!faceCascade.load([faceCascadePath UTF8String])) {
        NSLog(@"Could not load face cascade: %@", faceCascadePath);
    }
    
    // Load the upper body cascade classifier
    if (!upperBodyCascade.load([upperBodyCascadePath UTF8String])) {
        NSLog(@"Could not load upper body cascade: %@", upperBodyCascadePath);
    }
    
    // Load the full body cascade classifier
    if (!fullBodyCascade.load([fullBodyCascadePath UTF8String])) {
        NSLog(@"Could not load full body cascade: %@", fullBodyCascadePath);
    }
    
    // Load the lower body cascade classifier
    if (!lowerBodyCascade.load([lowerBodyCascadePath UTF8String])) {
        NSLog(@"Could not load lower body cascade: %@", lowerBodyCascadePath);
    }
    
    // FPS initialization
    weightRatio = 0.1;
    startTime = 0;
    framesInSecond = 0;
}

// Release outlets when the view is unloaded
- (void)viewDidUnload
{
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
    self.reader = nil;
    self.assetReaderOutput = nil;
    self.inputAsset = nil;
    self.videoFile = nil;
    self.fpsLabel = nil;
    self.groundTruth = nil;
}

// Dealloc outlets and class vars
- (void)dealloc {
    [self.reader release];
    [self.assetReaderOutput release];
    [self.inputAsset release];
    [self.videoFile release];
    [self.fpsLabel release];
    [self.groundTruth release];
    [super dealloc];
}

// Autorotate view
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}


#pragma mark - Image conversion

// This methods converts the sampleBuffer parameter to cv::Mat image format 
// and store it into the frame attribute
- (void)createMatImageFromSampleBuffer:(CMSampleBufferRef)sampleBuffer {
    if (sampleBuffer) { // if not null
        // Get the CVImageBufferRef
        CVImageBufferRef imageBuffer = CMSampleBufferGetImageBuffer(sampleBuffer);
        // Lock base address
        CVPixelBufferLockBaseAddress(imageBuffer, 0);
        
        // Get information of the image in the buffer
        void *bufferBaseAddress = CVPixelBufferGetBaseAddress(imageBuffer);
        size_t bufferWidth = CVPixelBufferGetWidth(imageBuffer);
        size_t bufferHeight = CVPixelBufferGetHeight(imageBuffer);
        
        // If the frame image hasn't been created yet, it's created not
        if (!frame.data) {
            frame = cv::Mat(bufferHeight, bufferWidth, CV_8UC4);
        }
        
        // If not null image, make the frame data point to bufferBaseAddres.
        if (bufferBaseAddress) {
            frame.data = (uchar*)bufferBaseAddress;
        }
        
        // Release memory
        CVPixelBufferUnlockBaseAddress(imageBuffer, 0);
    }    
}


// This methods creates a CGImage from a cv::Mat image
- (CGImageRef)createCGImageFromMatImage:(cv::Mat&)image {
    CGColorSpaceRef colorSpace;
    CGImageRef imageRef;
    
    if (image.channels() > 1) {
        // If channels is bigger than 1, it's a color image
        colorSpace = CGColorSpaceCreateDeviceRGB();
        CGDataProviderRef provider = CGDataProviderCreateWithData(NULL, image.data, image.elemSize()*image.total(), NULL);
        
        // Create CgImageRef from cgcontext
        imageRef = CGImageCreate(image.cols, image.rows, 8, image.elemSize()*8, image.step[0],colorSpace, kCGImageAlphaNoneSkipFirst | kCGBitmapByteOrder32Little,provider, NULL, false, kCGRenderingIntentDefault);
        
        CGDataProviderRelease(provider);
    } else {
        // If channel = 1, it's a grayscale image
        colorSpace = CGColorSpaceCreateDeviceGray();
        
        CGContextRef newContextGray = CGBitmapContextCreate(image.data, image.size().width, image.size().height, 8, image.size().width, colorSpace, kCGImageAlphaNone);
        
        // Create CgImageRef from cgcontext
        imageRef = CGBitmapContextCreateImage(newContextGray);
    }
    
    CGColorSpaceRelease(colorSpace);
    
    return imageRef;
}

#pragma mark - FPS
// Calculate the current FPS 
- (void)calculateFPS {
    
    // Frames processed since the last calculation
    framesInSecond++;
    
    // Time since the last calculation
    endTime = [[NSDate date] timeIntervalSince1970];
    
    // If it's the first time the FPS is calculated, we have to initialize 
    // the startTime variable
    if (startTime <= 0) {
        startTime = [[NSDate date] timeIntervalSince1970];
    } else {
        if (timeFrame > 0) {
            timeFrame = ((endTime - startTime) * (1.0 - weightRatio)) + (timeFrame * weightRatio);
            fpsAverage = 1 / timeFrame;
            startTime = [[NSDate date] timeIntervalSince1970]; // New star of the time interval
            framesInSecond = 0;
        } else {
            timeFrame = (endTime - startTime); // Processing time for the current frame
            fpsAverage = 1 / timeFrame;
            startTime = [[NSDate date] timeIntervalSince1970]; // New start of the time interval
            framesInSecond = 0;
        } 
    }
    
    // Update the FPS label in the view
    [self.fpsLabel performSelectorOnMainThread:@selector(setText:) withObject:[NSString stringWithFormat:@"FPS: %.2f", fpsAverage] waitUntilDone:YES];
}


#pragma mark - Video Displaying

// Call the OpenCV detection method and display the results
- (void)showFrame: (CMSampleBufferRef) sample {
    
        
    if (sample) {

        // Convert the CMSampleBufferRef image to a cv::Mat image
        [self createMatImageFromSampleBuffer:sample];
        
        // If the gray image hasn't been created yet, it must be created now
        if (!gray.data) {
            gray = cv::Mat(frame.size().height, frame.size().width, CV_8U);
        }
                
        // NEON SIMD color to grayscale conversion
        neon_to_gray(frame,gray);
        
        
        /* Uncomment arrays as needed */
        
        // Array with detected faces
        std::vector<cv::Rect> faces;
        
        // Array with detected upper bodys
        //std::vector<cv::Rect> upperBodys;
        
        // Array with detected lower bodys
        //std::vector<cv::Rect> lowerBodys;
        
        // Array with detected full bodys
        //std::vector<cv::Rect> fullBodys;
        
        
        /* Uncomment method callings as needed */
        
        // Face detection
        faceCascade.detectMultiScale(gray, faces, 1.1, 2, 0 | CV_HAAR_DO_CANNY_PRUNING, cv::Size(20, 20));
        
        // Upper body detection
        //upperBodyCascade.detectMultiScale(gray,upperBodys,1.1,2,0 | CV_HAAR_DO_CANNY_PRUNING, cv::Size(30,30));
        
        // Lower body detection
        //lowerBodyCascade.detectMultiScale(gray,lowerBodys,1.1,2,0 | CV_HAAR_DO_CANNY_PRUNING, cv::Size(30,30));
        
        // Full body detection
        //fullBodyCascade.detectMultiScale(gray,fullBodys,1.1,2, 0 | CV_HAAR_DO_CANNY_PRUNING, cv::Size(60,60));
        
        // Update the current frame number
        frameNumber++;
        
        // Draw a rectangle for each object detected. Uncomment loops as needed.
        
        // Draw faces detected
        for (int i = 0; i < faces.size(); i++) {            
            cv::rectangle(frame, cv::Point(faces[i].x,faces[i].y), cv::Point(faces[i].x+faces[i].width,faces[i].y+faces[i].height), cv::Scalar(255,0,0)); 
            //NSLog(@"%d %d %d %d %d",frameNumber,faces[i].x,faces[i].y,faces[i].width,faces[i].height);
        }
        
        // Draw upper bodys detected
        /*for (int i = 0; i < upperBodys.size(); i++) {
            cv::rectangle(frame, cv::Point(upperBodys[i].x,upperBodys[i].y), cv::Point(upperBodys[i].x+upperBodys[i].width,upperBodys[i].y+upperBodys[i].height), cv::Scalar(255,0,0));
            //NSLog(@"Frame %d %d %d %d %d",frameNumber,upperBodys[i].x,upperBodys[i].y,upperBodys[i].width,upperBodys[i].height);
        }*/

        // Draw lower bodys detected
        /*for (int i = 0; i < lowerBodys.size(); i++) {
         cv::rectangle(frame, cv::Point(lowerBodys[i].x,lowerBodys[i].y), cv::Point(lowerBodys[i].x+lowerBodys[i].width,lowerBodys[i].y+lowerBodys[i].height), cv::Scalar(0,0,255));
         //NSLog(@"Frame %d %d %d %d %d",frameNumber,lowerBodys[i].x,lowerBodys[i].y,lowerBodys[i].width,lowerBodys[i].height);
         }*/
        
        // Draw full bodys detected
        /*for (int i = 0; i < fullBodys.size(); i++) {
         cv::rectangle(frame, cv::Point(fullBodys[i].x,fullBodys[i].y), cv::Point(fullBodys[i].x+fullBodys[i].width,fullBodys[i].y+fullBodys[i].height), cv::Scalar(255,0,0));
         //NSLog(@"Frame %d %d %d %d %d",frameNumber,fullBodys[i].x,fullBodys[i].y,fullBodys[i].width,fullBodys[i].height);
         }*/
        
        
        
        // Draw the ground truth objects
        NSString *key = [NSString stringWithFormat:@"%d",frameNumber];
        
        Frame *f = [self.groundTruth.frames objectForKey:key];
        
        for (Detection *d in f.detections) {
            cv::rectangle(frame, cv::Point(d.x,d.y), cv::Point(d.x+d.width,d.y+d.height), cv::Scalar(0,255,0));
        }
        
        [f release];
        
        // We need to convert the cv::Mat image to a CGImageRef so it can be displayed by the view
        CGImageRef imageRef = [self createCGImageFromMatImage:frame];
        
        // Display image in the view
        [self.view.layer performSelectorOnMainThread:@selector(setContents:) withObject:(id)imageRef waitUntilDone:YES];
        
        CGImageRelease(imageRef);
        CFRelease(sample); 
        
        // FPS calc
        [self calculateFPS];

    }     
}

// Reads and process every frame in the video
- (void) displayProcessedVideo
{
    CMSampleBufferRef sample;
    
    dispatch_queue_t playingThread = dispatch_queue_create("playingThread", DISPATCH_QUEUE_SERIAL);
    
    // Total time interval (for FPS average)
    double start = [[NSDate date] timeIntervalSince1970];  
    
    double n=0;
    
    // while video is not finished
    while ( [self.reader status]==AVAssetReaderStatusReading ) {
        
        // get the next frame
        sample = [self.assetReaderOutput copyNextSampleBuffer];
        
        // display the frame in another thread
        dispatch_sync(playingThread, ^(void){
            [self showFrame:sample];
        });  
        
        n++;
    }	
    
    
    // At the end of the video, the average FPS is displayed
    double finish = [[NSDate date] timeIntervalSince1970];   
    double fps_total = n / (finish - start);

    [self.fpsLabel performSelectorOnMainThread:@selector(setText:) withObject:[NSString stringWithFormat:@"FPS(AVG): %.2f", fps_total] waitUntilDone:YES];
    
    dispatch_release(playingThread);
}


// Creates and sets up an AVAssetReader object to read the video file
- (void) playVideo
{
    if (self.videoFile != nil) {
        // input asset options
        NSDictionary *inputOptions = [NSDictionary dictionaryWithObject:[NSNumber numberWithBool:YES] forKey:AVURLAssetPreferPreciseDurationAndTimingKey];
    
        // init the input asset with that options
        self.inputAsset = [[AVURLAsset alloc] initWithURL:[NSURL fileURLWithPath:self.videoFile] options:inputOptions];
    
        // when the asset is loaded the following code is executed in another thread
        [self.inputAsset loadValuesAsynchronouslyForKeys:[NSArray arrayWithObject:@"tracks"] completionHandler:^(void) {
                
            // init the asset reader
            self.reader = [[AVAssetReader alloc] initWithAsset:inputAsset error:NULL];     
        
            // get the video track
            NSArray* video_tracks = [inputAsset tracksWithMediaType:AVMediaTypeVideo]; 
            AVAssetTrack* video_track = [video_tracks objectAtIndex:0];
        
            // image format
            NSMutableDictionary* dictionary = [[NSMutableDictionary alloc] init];
            [dictionary setObject:[NSNumber numberWithInt:kCVPixelFormatType_32BGRA] forKey:(NSString*)kCVPixelBufferPixelFormatTypeKey];
        
            // image output asset
            self.assetReaderOutput = [[AVAssetReaderTrackOutput alloc] initWithTrack:video_track outputSettings:dictionary];
        
            [self.reader addOutput:self.assetReaderOutput];
        
            // start reading frames
            [self.reader startReading];
            
            frameNumber = 0;
        
            // display frames
            [self displayProcessedVideo];
        }];
    }
}

@end
